      SUBROUTINE NAVE
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/WAVE/CABL,CNBL,CMBL,CAW,CNW,CMW
      COMMON/DIS2/SUM1,SUM2,SUM3,SUM4,SUM5,SUM6
      COMMON/CVP/CXT(240),CNT(240),CMT(240),CPV(21),JA,JB,KF
      COMMON/RX/XR(20,6),G(20),GCH,DM,N,N1(20),IX,I1
      COMMON/NNI/NTT(11),NP5,JZT
      COMMON/GEO3/VOVS,AL,YINT,F,RR,RREF,AREF,DLN
      CAW=0.D0
      CNW=0.D0
      CMW=0.D0
      NK1=N-1
      DO 1 J=1,NK1
      JA=NTT(J)+1
      JB=NTT(J+1)
      CALL SIMP
      CMW=CMW+2.D0*SUM3/(AREF*2.D0*RREF)
      CAW=CAW+2.D0*SUM1/AREF
      CNW=CNW-2.D0*SUM2/AREF
  1   END DO
      CAW=CAW+CABL
      CNW=CNW+CNBL
      CMW=CMW+CMBL
      RETURN
      END
