      SUBROUTINE NEWT
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/DISC/R1,PI,I,JH,J,KL
      COMMON/WAVE/CABL,CNBL,CMBL,CAW,CNW,CMW
      COMMON/CVP/CXT(240),CNT(240),CMT(240),CPV(21),JA,JB,KF
      COMMON/GEO2/NFL,NN,IPRINT,MAL,MAX,IPR
      COMMON/GEOM1/XB(240),RB(240),RBP(240),C2,BETA
      COMMON/NNI/NTT(11),NP5,JZT
      COMMON/GEO3/VOVS,AL,YINT,F,RR,RREF,AREF,DLN
      PLPI=(1.2D0*VOVS**2)**3.5D0*(6.D0/(7.D0*VOVS**2-1.D0))**2.5
      IF(NFL.EQ.1)GO TO 2
      CPO=0.999D0*(PLPI-1.0D0)/(0.7D0*VOVS**2)
      IF(IPRINT.NE.1)GO TO 19
      WRITE(2,6)CPO
   6  FORMAT(1X,45HKO|fficiEHT DABlEHiq Dlq plOCKOgO zATYplEHiq=,F10.5)
  19  CA=CPO*(RR/RREF)**2
      CABL=CA
      CNBL=0.D0
      CMBL=0.D0
      IF(JZT.EQ.0) RETURN
      GO TO 18
   2  CPO=(PLPI-1.D0)/(0.7D0*VOVS**2)
      IF(IPRINT.NE.1)GO TO 18
      PRINT 8,CPO
   8  FORMAT(1X,25HKO|fficiEHT dABlEHiq dlq ,24HCfEPi~ECKOgO zATYplEHiq
     *,F10.5)
   18 AR=AL
      IF(AR.EQ.0.D0) AR=1.0D-7
      DF=PI/(KF-1.)
      CSA=DCOS(AR)
      SNA=DSIN(AR)
      TNA=SNA/CSA
      I1=KF/2
      DO 3 I=JA,JB
      TNT=RBP(I)
      TET=DATAN(TNT)
      SNT=DSIN(TET)
      CST=DCOS(TET)
      SNFU=TNT/TNA
      P1=CSA*SNT
      P2=SNA*CST
      DO 1 J=1,KF
      SNF=DSIN((I1+1-J)*DF)
      CPV(J)=CPO*(P1-P2*SNF)**2
      IF(SNF.GT.SNFU) CPV(J)=0.D0
  1   END DO
      CALL POPIN
 3    END DO
      RETURN
      END
